# Author: Oriol Filter
# Date: 06/04/2022


$DebugPreference = "Continue"
$ErrorActionPreference = "Stop"
$SCRIPT_PATH=$PSScriptRoot
$HYPERV_FILE = "$SCRIPT_PATH/hyperv_list.txt"
$SCVMM_FILE = "$SCRIPT_PATH/hyperv_list.txt"
$HOSTS_FILE = "$SCRIPT_PATH/hosts.txt"
## Objects


## Add Vhost objects for each type of server

class VirtualizationServer {
    [int]$port=8100
    [string]$url=""
    #     [Boolean] _FindHost ($vname) {
    #         $type=$this.GetType().BaseType
    #         Write-Host "Find Host not implemented in type $type"
    #         return $false
    #     }
    #     [Boolean] FindHost ($vname) {
    #         return $this._FindHost($vname)
    #     }

    __load_hosts(){
        $type=$this.GetType().BaseType
        Write-Warning "Load hosts not implemented in type $type"
    }

    load_hosts(){
        $this.__host_list=@()
        $this.__load_hosts()
    }

    [Array] $__host_list=@()
    [Array] get_host_list(){
        return $this.__host_list
    }
}
class SystemCenterVirtualMachineManager: VirtualizationServer {
    __load_hosts(){
        Get-SCVMMServer -ComputerName $this.url
        $list = Get-SCVirtualMachine
        $this.__host_list=$list
        [string]::Format("Loaded VM from {0}",$this.url) | Write-Debug

    }
}
class HypervisorServer: VirtualizationServer{
    #     [Boolean] _FindHost ($vname) {
    #         $result=$false
    #         try
    #         {
    #             Invoke-Command -ComputerName $this.url {
    #                 Param ($name) Get-VM -Name $name
    #             } -ArgumentList $vname
    #             $result=$?
    #         }
    #         catch
    #         {
    #             Format ("{0}: {1}",$this.url,$Error[0]) | Write-Debug
    #         }
    #
    #         return $result
    #     }
}
class VMWareServer: VirtualizationServer{}

class VirtualizerManager{
    [Array]$__connected_servers=@()
    [Hashtable]$__found_host_dict=@{}
    [Boolean] __log_in($url){
        $status_message=""
        $result=$false
        try {
            $result=Invoke-Command -ComputerName $url { $true }
            $status_message = "OK"
        }
        catch [System.Management.Automation.Remoting.PSRemotingTransportException] {
            $status_message = Format("Error '{0}'",$_.Exception.GetType().fullname)
        }
        finally {
            $message = [string]::Format(">> [{0}] {1}",$url,$status_message)
            Write-Debug $message
        }
        return $result
    }
    hidden [Hashtable] get_found_host_dict() {return $this.__found_host_dict}

    append_session($type,$url)
    {
        $type.GetType().IsValueType
        $result = $this.__log_in($url)
        if ($result)
        {
            $server=$type::new()
            $server.url=$url
            $this.__connected_servers+=$server
        }
    }
    [Array] connected_servers() {
        return $this.__connected_servers
    }

    load_hosts() {
        $this.__found_host_dict=@{}
        for ($num = 0 ; $num -le $this.__connected_servers.Length -1 ; $num++){
            $server = $this.__connected_servers[$num]
            $server.load_hosts()
            foreach ($vhost in $server.get_host_list())
            {
#                $item.name | Write-Host
                $this.append_vhost($vhost,$server.url)
            }
        }
    }
    __append_vhost($vhost,$source_url){
        if ($this.__found_host_dict[$vhost.Name] -isnot [Hashtable]){
            $this.__found_host_dict[$vhost.Name]=@{}
        }
        $this.__found_host_dict[$vhost.Name.ToLower()][$source_url]=$vhost
    }
    append_vhost($vhost,$source_url){
        $this.__append_vhost($vhost,$source_url)
    }
    find_hosts($vname){
        $hosts_to_find=$vname
        if ($hosts_to_find -isnot [Array]){
            $hosts_to_find=@("$hosts_to_find")
        }
        # Find hosts
        foreach ($hostname  in $hosts_to_find) {
            $lhostname=$hostname.ToLower()
            $entry_from_list=$this.__found_host_dict[$lhostname]
            [String]::Format("`t{0}:",$hostname) | Write-Host
#            $item_from_list |Write-Host
#            foreach ($a in $item_from_list) {Write-Host $a}
#            $this.found_host_dict[$vhostname.ToString()].Lenght | Write-Host
#
#            $item_from_list.Length | Write-Host
            $entry_from_list.Length | Write-Host
            Write-Host "IF"
            if ($entry_from_list.Length > 0)
            {
                Write-Host "Inner IF"
                ## Not being executed?
                Write-Host "IF"
                Write-Host "!!"
                foreach ($server_location  in $entry_from_list) {
                    {
                        $server_location | Write-Host
                        $vm = $this.__found_host_dict[$lhostname][$server_location]
                        #$vm | Write-Host
                        [String]::Format("{0}:",$server_location) | Write-Host
                    }
                }
                else {
                    [String]::Format("---- Couldn't be found") | Write-Host
                }
            }
        }

    }
    #     find_host($vname){
    #         $hostlist=$vname
    #         if ($hostlist -is [Array]){
    #             $hostlist=@($hostlist)
    #         }
    #         foreach ($_vname in $hostlist)
    #         {
    #             for ($num = 0 ; $num -le $this.__connected_servers.Length -1 ; $num++){
    #                     $server= $this.__connected_servers[$num]
    #                     $result = $server.FindHost($_vname)
    #                     [String]::Format(">> {0} found in {1}? {2}", $_vname, $server.url, $result) | Write-Host
    #             }
    #             }
    #         }
}

function Get-Hosts-From-File($filepath)
{
    ## In a future use a yaml as a .conf (with the hyperv and vmware listed)
    $file_content = Get-Content "$filepath"
    $splitted_file_content = $file_content.Split()
    $host_array=@()
    foreach ($hostname in $splitted_file_content)
    {
        if ($hostname)
        {
            $host_array+=$hostname
        }
    }
    return $host_array
}


# Advanced params?
# https://docs.microsoft.com/en-us/powershell/module/microsoft.powershell.core/about/about_functions_advanced_parameters?view=powershell-7.2
function load_hyperv($manager) {
    $hyperv_url = Get-Hosts-From-File($HYPERV_FILE)
    foreach ($url in $hyperv_url) {
        $manager.append_session([HypervisorServer], $url)
    }
}

function load_scvmm($manager) {
    $hyperv_url = Get-Hosts-From-File($SCVMM_FILE)
    foreach ($url in $hyperv_url) {
        $manager.append_session([SystemCenterVirtualMachineManager], $url)
    }
}

function Main{
    $manager = [VirtualizerManager]::new()
    load_scvmm($manager)
    $manager.load_hosts()
    $vmhosts = Get-Hosts-From-File($HOSTS_FILE)

#    $manager.get_found_host_dict()
#    foreach ($hostname  in $manager.get_found_host_dict()){
#        Write-Output $hostname.Name
#    }
    $manager.__found_host_dict.Lenght
    $manager.find_hosts($vmhosts)
}

function Test_VCSMM{
    $manager = [VirtualizerManager]::new()
    load_scvmm($manager)
    [VirtualizationServer]$x=$manager.__connected_servers[0]
    $x.load_hosts()
    $x.get_host_list()

}
#Test_VCSMM
Main
